/************************************************************************************
*                                                                                   *
*   File Name   : extints.c                                                            *
*   Contents    : functions related with the EXTINT's 0,1,2 for the Energy Monitor                     *
*   Version     : 1.6                                                               *
*************************************************************************************/ 
#include "sfr_r813.h"                       /* Definition of the R8C/10 SFR */
#include "funcs.h"
#include <stdlib.h>

/************************************************************************************
Name:        EXTINT0_init
Parameters:  None
Returns:     None
Description: Initial setting of EXTINT0 registers
************************************************************************************/
void EXTINTS_init(void)
{
	/* Setting port registers */
    p1 = p1 | 0x07;                         /* P1.2-P1.0 = H(Led 3.2.1) P1.3 = L (Led 4) */
    /* Setting port direction registers */
    pd1 = pd1 | 0x0f;                       /* P1.3-P1.0 port direction = output */
	/* settings for INT0 */
	int0ic        = 0x00; /* interrupt disabled due to prio level 0 */
	pd4           = pd4 & 0xdf; /* P4.5 = INT0 is input */
	pu11          = 0;    /* P4.5 not pulled-up */
	int0en        = 1;    /* interrupt INT0 enabled */
	int0pl 		  = 0;    /* just 1 edge counted */
	int0f         = 0;    /* no filter */
	pol_int0ic    = 0;    /* selects falling edge */
	ir_int0ic     = 0;    /* clear interrupt request flag */
	ilvl0_int0ic  = 1;
	ilvl1_int0ic  = 1;
	ilvl2_int0ic  = 1;	  /* interrupt prio level7 */

	/* settings for INT1 */
	int1ic        = 0x00; /* interrupt disabled due to prio level 0 */
	pd1           = pd1 & 0x7f; /* P1.7 = INT1 is input */
	pu03          = 0;    /* P1.4..P1.7 not pulled-up */
	r0edg         = 0;    /* selects falling edge */
	ir_int1ic     = 0;    /* clear interrupt request flag */
	ilvl0_int1ic  = 1;
	ilvl1_int1ic  = 1;
	ilvl2_int1ic  = 1;	  /* interrupt prio level7 */

	/* settings for INT2 */
	int2ic        = 0x00; /* interrupt disabled due to prio level 0 */
	pd3           = pd3 & 0xfd; /* P3.2 = INT2 is input */
	pu06          = 0;    /* P3.0..P3.3 not pulled-up */
	r1edg         = 0;    /* selects falling edge */
	ir_int2ic     = 0;    /* clear interrupt request flag */
	ilvl0_int2ic  = 1;
	ilvl1_int2ic  = 1;
	ilvl2_int2ic  = 1;	  /* interrupt prio level7 */
}

/************************************************************************************
Name:        interrupt Extint 0
Parameters:  None
Returns:     None
Description: ISR for EXTINT0 (electric Consumption)
************************************************************************************/
#pragma interrupt Extint_0_int
void Extint_0_int(void)
{
long diff;
	if (ini_rcvd_yet)
	{
		diff = ui_TIRQ_count10ms - ui_EC_EIRQ_timestamp;
		if (labs(diff) > cConsecutiveEventsElecTime) /* 100ms filter time for smallest time between 2 events */
		{
			ui_EC_EIRQ_timestamp = ui_TIRQ_count10ms; /* update timestamp of sucessfull IRQ */
		} //if (abs(ui_EC_EIRQ_timestamp - ui_TIRQ_count10ms) > 30)
	} //if (ini_rcvd_yet)
}

/************************************************************************************
Name:        interrupt Extint 1
Parameters:  None
Returns:     None
Description: ISR for EXTINT1 (gas Consumption)
************************************************************************************/
#pragma interrupt Extint_1_int
void Extint_1_int(void)
{
long long diff;
	if (ini_rcvd_yet)
	{
		diff = ui_TIRQ_count10ms - ui_GC_EIRQ_timestamp;
		if (labs(diff) > cConsecutiveEventsGasTime) /* 100ms filter time for smallest time between 2 events */
		{
			ui_GC_EIRQ_timestamp = ui_TIRQ_count10ms; /* update timestamp of sucessfull IRQ */
		} //if (abs(ui_GC_EIRQ_timestamp - ui_TIRQ_count10ms) > 30)
	} //if (ini_rcvd_yet)
}

/************************************************************************************
Name:        interrupt INT2
Parameters:  None
Returns:     None
Description: ISR for EXTINT2 (water Consumption)
************************************************************************************/
#pragma interrupt Extint_2_int
void Extint_2_int(void)
{
long long diff;
	if (ini_rcvd_yet)
	{
		diff = ui_TIRQ_count10ms - ui_WC_EIRQ_timestamp;
		if (labs(diff) > cConsecutiveEventsWaterTime) /* 100ms filter time for smallest time between 2 events */
		{
			ui_WC_EIRQ_timestamp = ui_TIRQ_count10ms; /* update timestamp of sucessfull IRQ */
		} //if (abs(ui_WC_EIRQ_timestamp - ui_TIRQ_count10ms) > 30)
	} //if (ini_rcvd_yet)
}

/***********************************************************************************/
