/************************************************************************************
*                                                                                   *
*   File Name   : main.c                                                            *
*   Contents    : Mainloop of the Energy Monitor                                    *
*   Version     : 1.11                                                               *
*************************************************************************************/ 
#include "sfr_r813.h"                       /* Definition of the R8C/10 SFR */
#include "funcs.h"

/* Definition of RAM area */
unsigned int ui_TIRQ_count10ms            = 0;

unsigned int  ui_EC_EIRQ_timestamp        = 0;
unsigned long ul_ML_electricalConsumption = 0; 
unsigned char uc_ML_pulsesElPow        = 0;
unsigned char uc_EC_TIRQ_validEvent       = 0;
unsigned char uc_EC_ML_validEvent         = 0;
unsigned int  ui_EC_TIRQ_timestamp        = 0;
unsigned int  ui_EC_TIRQ_debounceCounter  = 0;

unsigned int  ui_GC_EIRQ_timestamp        = 0;
unsigned long ul_ML_gasConsumption        = 0;
unsigned char uc_GC_TIRQ_validEvent       = 0;
unsigned char uc_GC_ML_validEvent         = 0;
unsigned int  ui_GC_TIRQ_timestamp        = 0;
unsigned int  ui_GC_TIRQ_debounceCounter  = 0; 

unsigned int  ui_WC_EIRQ_timestamp        = 0;
unsigned long ul_ML_waterConsumption      = 0;
unsigned char uc_ML_pulsesWater           = 0;
unsigned char uc_WC_TIRQ_validEvent       = 0;
unsigned char uc_WC_ML_validEvent         = 0;
unsigned int  ui_WC_TIRQ_timestamp        = 0;
unsigned int  ui_WC_TIRQ_debounceCounter  = 0;

rs232_rcv_type rs232_rcv;
unsigned char rs232_rcv_array[36];
unsigned char startIndicator            = 0;
unsigned char positionPointer           = 0;
_Bool         b_VERSIONsendREQ          = 0;

unsigned char ini_rcvd_yet              = 0;
unsigned int  ui_ML_waitCnt;


#define convRcvArray(x) ((rs232_rcv_array[x])-0x30)

/* Declaration of function prototype */

void toggle_lower3leds (void)
{
	cEC_PORT = ~cEC_PORT;
	cGC_PORT = ~cGC_PORT;
	cWC_PORT = ~cWC_PORT;
}

/* >>> the real code
void f_ML_setECvalues(void)
{
	if (uc_ML_pulsesElPow < (cPULSES_PER_0_01KWH-1))
	{
		uc_ML_pulsesElPow++;
	} //if (uc_ML_pulsesElPow < cPULSES_PER_0_01KWH)
	else
	{
		//reset inner counter
		uc_ML_pulsesElPow = 0;
		//increase counter value
		if (ul_ML_electricalConsumption > 99999999)
		{
			ul_ML_electricalConsumption = 0;
		}
		else
		{
			ul_ML_electricalConsumption++;
		}
	} //else [if (uc_ML_pulsesElPow < cPULSES_PER_0_01KWH)]
	//toggle LED Px.0
	cEC_PORT = ~cEC_PORT;
} <<< the real code
*/

void f_ML_setECvalues(void)
{
	//increase counter value
	if (ul_ML_electricalConsumption > 99999999)
	{
		ul_ML_electricalConsumption = 0;
	}
	else
	{
 		ul_ML_electricalConsumption++;
	}
	//toggle LED Px.0
	cEC_PORT = ~cEC_PORT;
}

void f_ML_setGCvalues(void)
{
	//increase counter value
	if (ul_ML_gasConsumption > 9999999)
	{
		ul_ML_gasConsumption = 0;
	}
	else
	{
		ul_ML_gasConsumption++;
	}
	//toggle LED Px.0
	cGC_PORT = ~cGC_PORT;
}

/* >>>the real code
void f_ML_setWCvalues(void)
{
	if (uc_ML_pulsesWater < (cPULSES_PER_0_00001M3-1))
	{
		uc_ML_pulsesWater++;
	} //if (uc_ML_pulsesWater < cPULSES_PER_0_00001M3)
	else
	{
		//reset inner counter
		uc_ML_pulsesWater = 0;
		//increase counter value
		if (ul_ML_waterConsumption > 999999999)
		{
			ul_ML_waterConsumption = 0;
		}
		else
		{
			ul_ML_waterConsumption++;
		}
	} //else [if (uc_ML_pulsesWater < cPULSES_PER_0_00001M3)]
	//toggle LED Px.0
	cWC_PORT = ~cWC_PORT;
} <<<the real code
*/
void f_ML_setWCvalues(void)
{
	//increase counter value
	if (ul_ML_waterConsumption > 999999999)
	{
		ul_ML_waterConsumption = 0;
	}
	else
	{
		ul_ML_waterConsumption++;
	}
	//toggle LED Px.0
	cWC_PORT = ~cWC_PORT;
}


main() {
    asm("FCLR I");                          /* Interrupt disable */

    /*-----------------------------------------------
    -Change on-chip oscillator clock to Main clock  -
    -------------------------------------------------*/
    prc0 = 1;                               /* Protect off */
    cm13 = 1;                               /* Xin Xout */
    cm15 = 1;                               /* XCIN-XCOUT drive capacity select bit : HIGH */
    cm05 = 0;                               /* Xin on */
    cm16 = 0;                               /* Main clock = No division mode */
    cm17 = 0;
    cm06 = 0;                               /* CM16 and CM17 enable */
    asm("nop");                             /* Waiting for stable of oscillation */
    asm("nop");
    asm("nop");
    asm("nop");
    ocd2 = 0;                               /* Main clock change */
    prc0 = 0;                               /* Protect on */

    /*-----------------------------------
    -   Initialize SFR                  -
    -----------------------------------*/
    
	TIMERX_init();                          /* Initial setting of TIMER X registers */

	UART0_init();							/* Initial setting of UART0 registers */

	EXTINTS_init();							/* Initial setting of EXTINT0 registers */

    asm("FSET I");                          /* Interrupt enable */

    /*-----------------------------------
    -   Loop of main                    -
    -----------------------------------*/
    while(1)
	{                               /* Main processing */
		asm("FCLR I");              /* Interrupt disable */
		/* electrical consumption data processing */
		if ( (uc_EC_TIRQ_validEvent == 1) &&
		     (uc_EC_ML_validEvent == 0)      )
        {
			uc_EC_ML_validEvent = 1;
			f_ML_setECvalues();
		}
		else if ( (uc_EC_TIRQ_validEvent == 0) &&
		          (uc_EC_ML_validEvent == 1)      )
		{
			uc_EC_ML_validEvent = 0;
		}
		/* -------------------------------------- */

		/* gas consumption data processing */
		if ( (uc_GC_TIRQ_validEvent == 1) &&
		     (uc_GC_ML_validEvent == 0)      )
        {
			uc_GC_ML_validEvent = 1;
			f_ML_setGCvalues();
		}
		else if ( (uc_GC_TIRQ_validEvent == 0) &&
		          (uc_GC_ML_validEvent == 1)      )
		{
			uc_GC_ML_validEvent = 0;
		}
		/* -------------------------------------- */
		
		/* water consumption data processing */
		if ( (uc_WC_TIRQ_validEvent == 1) &&
		     (uc_WC_ML_validEvent == 0)      )
        {
			uc_WC_ML_validEvent = 1;
			f_ML_setWCvalues();
		}
		else if ( (uc_WC_TIRQ_validEvent == 0) &&
		          (uc_WC_ML_validEvent == 1)      )
		{
			uc_WC_ML_validEvent = 0;
		}
		/* -------------------------------------- */
		
		if (positionPointer == 35)
		{
		   ul_ML_electricalConsumption = (unsigned long)convRcvArray(0)*10000000 + (unsigned long)convRcvArray(1)*1000000 + (unsigned long)convRcvArray(2)*100000 + (unsigned long)convRcvArray(3)*10000 + (unsigned long)convRcvArray(4)*1000 + (unsigned long)convRcvArray(5)*100 + (unsigned long)convRcvArray(7)*10 + (unsigned long)convRcvArray(8);	
		   ul_ML_gasConsumption        = (unsigned long)convRcvArray(12)*1000000 + (unsigned long)convRcvArray(13)*100000 + (unsigned long)convRcvArray(14)*10000 + (unsigned long)convRcvArray(15)*1000 + (unsigned long)convRcvArray(16)*100 + (unsigned long)convRcvArray(18)*10 + (unsigned long)convRcvArray(19);
		   ul_ML_waterConsumption      = (unsigned long)convRcvArray(23)*100000000 + (unsigned long)convRcvArray(24)*10000000 + (unsigned long)convRcvArray(25)*1000000 + (unsigned long)convRcvArray(26)*100000 + (unsigned long)convRcvArray(27)*10000 + (unsigned long)convRcvArray(29)*1000 + (unsigned long)convRcvArray(30)*100 + (unsigned long)convRcvArray(31)*10 + (unsigned long)convRcvArray(32);
		   positionPointer = 0;
		   toggle_lower3leds();        /* quit sucessfull reception with toggling lower 3 LED's */
		}
		asm("FSET I");              /* Interrupt enable */

		//150s (100 Nop's) wait time with interrupts enabled
		for (ui_ML_waitCnt=0;ui_ML_waitCnt<cMaxWaitCycles;ui_ML_waitCnt++)
		{
			asm("nop");
		}
	} //while (1)
}
