/************************************************************************************
*                                                                                   *
*   File Name   : timer_x.c                                                         *
*   Contents    : Timer X related functions of the Energy Monitor                   *
*   Version     : 1.4                                                               *
*************************************************************************************/ 
#include "sfr_r813.h"                       /* Definition of the R8C/10 SFR */
#include "funcs.h"


/************************************************************************************
Name:        TIMERX_init
Parameters:  None
Returns:     None
Description: Initial setting of Timer X registers
************************************************************************************/
void TIMERX_init(void)
{
    txck0 = 0;                              /* Timer X count source = f1 */
    txck1 = 0;

    /* Setting the main cycle timer                   */
    /* 20 MHz * 1/32 * 1/25 * 1/250 => 100 Hz = 10ms  */
    prex = 25-1;                            /* Setting Prescaler X register */
    tx   = 250-1;                           /* Setting timer X register */

    txck0	= 0;                     		/* Timer Count source = f32 */
    txck1	= 1;
	
	txmr = 0x00;                            /* Timer X : timer mode */
    txic = 5;                               /* Interrupt priority level = 5 */

    ui_TIRQ_count10ms = 0;                    /* 1 second counter initialization */

    ir_txic = 0;                            /* Interrupt request flag clear */
    txs  = 1;                               /* Timer X count start flag = start */
}
/************************************************************************************
Name:        interrupt Timer X
Parameters:  None
Returns:     None
Description: Timer X
************************************************************************************/
#pragma interrupt Timer_X_int
void Timer_X_int(void)
{
    ui_TIRQ_count10ms++;                          /* 10 msecond counter increment */

    /* electrical consumption data processing */
	if (uc_EC_ML_validEvent)
	{
		uc_EC_TIRQ_validEvent = 0; //reset valid event indicator if accepted from main loop
	}
	if( ui_EC_EIRQ_timestamp != ui_EC_TIRQ_timestamp ) //a new event from Extint0 to be processed
	{
		ui_EC_TIRQ_debounceCounter = cDebounceTimeElec;  //debounce counter of 30ms
		ui_EC_TIRQ_timestamp = ui_EC_EIRQ_timestamp; //quit event to prevent again entering
	}
	else if ( ui_EC_TIRQ_debounceCounter > 0) //check debounce counter underflow
	{
		ui_EC_TIRQ_debounceCounter--; //decrement  debounce counter
	}
	else if ( (ui_EC_TIRQ_debounceCounter == 0) &&
	          (cEC_SENS_PORT == 0) // check that Extint0-pin is low after debounce time
	        )
	{
		uc_EC_TIRQ_validEvent = 1; //indicate finding of valid event to main loop
	}
	/* -------------------------------------- */
	
	
	/* gas consumption data processing */
	if (uc_GC_ML_validEvent)
	{
		uc_GC_TIRQ_validEvent = 0; //reset valid event indicator if accepted from main loop
	}
	if( ui_GC_EIRQ_timestamp != ui_GC_TIRQ_timestamp )
	{
		ui_GC_TIRQ_debounceCounter = cDebounceTimeGas; //debounce counter of 100ms
		ui_GC_TIRQ_timestamp = ui_GC_EIRQ_timestamp;
	}
	else if ( ui_GC_TIRQ_debounceCounter > 0)
	{
		ui_GC_TIRQ_debounceCounter--;
	}
	else if ( (ui_GC_TIRQ_debounceCounter == 0) &&
	          (cGC_SENS_PORT == 0)
	        )
	{
		uc_GC_TIRQ_validEvent = 1;
	}
	/* -------------------------------------- */
	
	/* water consumption data processing */
	if (uc_WC_ML_validEvent)
	{
		uc_WC_TIRQ_validEvent = 0; //reset valid event indicator if accepted from main loop
	}
	if( ui_WC_EIRQ_timestamp != ui_WC_TIRQ_timestamp )
	{
		ui_WC_TIRQ_debounceCounter = cDebounceTimeWater; //debounce counter of 70ms
		ui_WC_TIRQ_timestamp = ui_WC_EIRQ_timestamp;
	}
	else if ( ui_WC_TIRQ_debounceCounter > 0)
	{
		ui_WC_TIRQ_debounceCounter--;
	}
	else if ( (ui_WC_TIRQ_debounceCounter == 0) &&
	          (cWC_SENS_PORT == 0)
	        )
	{
		uc_WC_TIRQ_validEvent = 1;
	}
	/* -------------------------------------- */


	if ( (ui_TIRQ_count10ms%100) == 0 )           /* 1 second have passed ? */
    {
        c1S_TICK_PORT = ~c1S_TICK_PORT;
		if (!(ini_rcvd_yet))
		{
			toggle_lower3leds();
		}
    }

	if ( ui_TIRQ_count10ms == 1000 )           /* 10 seconds have passed ? */
    {
		ui_TIRQ_count10ms = 0;				/* 10 msecond counter clear */	
		sendConsumptionValues(ul_ML_electricalConsumption, ul_ML_gasConsumption, ul_ML_waterConsumption);
	}
}
