/************************************************************************************
*                                                                                   *
*   File Name   : uart.c                                                            *
*   Contents    : UART-related functions for the Energy Monitor                     *
*   Version     : 1.6                                                               *
*************************************************************************************/ 
#include "sfr_r813.h"                       /* Definition of the R8C/10 SFR */
#include "funcs.h"

/************************************************************************************
Name:        sendTxd0
Parameters:  unsigned char data - byte to be send via UART0
Returns:     None
Description: Initial setting of UART0 registers
************************************************************************************/
void sendTxd0(unsigned char data)
{
	while (ti_u0c1 == 0);
	u0tbl = data;
	te_u0c1 = 1;
}

/************************************************************************************
Name:        UART0_init
Parameters:  None
Returns:     None
Description: Initial setting of UART0 registers
************************************************************************************/
void UART0_init(void)
{
	p1 = p1 | 0x10; 
	pd1 = pd1 | 0x10;
	pd1 = pd1 & 0xdf;
	u0mr = 0x05;
	u0c0 = 0x00;
	u0rrm = 0;
	u0brg = 130-1;
	ilvl0_s0ric = 0;
	ilvl1_s0ric = 0;
	ilvl2_s0ric = 1; /* interrupt prio 4 */	
	re_u0c1 = 1;
	ir_s0ric = 0; /* clear reception flag */
}

/************************************************************************************
Name:        sendConsumptionValues
Parameters:  unsigned long electricalConsumption - value which contains the electrical power consumption in kWH with 6 digits
             unsigned long gasConsumption        - value which contains the gas consumption in m^3 with 5 digits before and 2 digits after decimal point
			 unsigned long waterConsumption      - value which contains the water consumption in m^3 with 5 digits
Returns:     None
Description: Initial setting of UART0 registers
************************************************************************************/
void sendConsumptionValues(unsigned long electricalConsumption, unsigned long gasConsumption, unsigned long waterConsumption)
{
extern unsigned int mk_month;
extern unsigned int mk_day;

unsigned char digit;
double calcValue;

sendTxd0('s'); /* send start character 's' */

if (b_VERSIONsendREQ)
{
   calcValue = ((double)mk_month / 10.0);
   digit     = (unsigned char)calcValue;
   sendTxd0(digit+0x30); /* send 10 digit of mk_month */
   calcValue = ((double)mk_month - (double)(digit * 10));
   digit     = (unsigned char)calcValue;
   sendTxd0(digit+0x30); /* send 1 digit of mk_month */
   calcValue = ((double)mk_day / 10.0);
   digit     = (unsigned char)calcValue;
   sendTxd0(digit+0x30); /* send 10 digit of mk_day */
   calcValue = ((double)mk_day - (double)(digit * 10));
   digit     = (unsigned char)calcValue;
   sendTxd0(digit+0x30); /* send 1 digit of mk_day */
   b_VERSIONsendREQ = 0;
}
else
{
   if (ini_rcvd_yet)
   {
	  calcValue = ((double)electricalConsumption / 10000000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 10.000.000 digit of electrical consumption */
	  electricalConsumption = (double)electricalConsumption - (double)((double)digit * 10000000.0);
	  calcValue = ((double)electricalConsumption / 1000000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 1.000.000 digit of electrical consumption */
	  electricalConsumption = (double)electricalConsumption - (double)((double)digit * 1000000.0);
	  calcValue = ((double)electricalConsumption / 100000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 100.000 digit of electrical consumption */
	  electricalConsumption = (double)electricalConsumption - (double)((double)digit * 100000.0);
	  calcValue = ((double)electricalConsumption / 10000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 10.000 digit of electrical consumption */
	  electricalConsumption = (double)electricalConsumption - (double)((double)digit * 10000.0);
	  calcValue = ((double)electricalConsumption / 1000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 1.000 digit of electrical consumption */
	  electricalConsumption = (double)electricalConsumption - (double)((double)digit * 1000.0);
	  calcValue = ((double)electricalConsumption / 100.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 100 digit of electrical consumption */
	  sendTxd0('.');
	  electricalConsumption = (double)electricalConsumption - (double)((double)digit * 100.0);
	  calcValue = ((double)electricalConsumption / 10.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 10 digit of electrical consumption */
	  electricalConsumption = (double)electricalConsumption - (double)((double)digit * 10.0);
	  calcValue = ((double)electricalConsumption / 1.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 1 digit of electrical consumption */
   } //if (ini_rcvd_yet)
   else
   { 
  	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('.');
	  sendTxd0('-');
	  sendTxd0('-');
   } //else of if (ini_rcvd_yet)
   sendTxd0('K');
   sendTxd0('W');
   sendTxd0('H');


   if (ini_rcvd_yet)
   {
	  calcValue = ((double)gasConsumption / 1000000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 1.000.000 digit of gas consumption */
	  gasConsumption = (double)gasConsumption - (double)((double)digit * 1000000.0);
	  calcValue = ((double)gasConsumption / 100000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 100.000 digit of gas consumption */
	  gasConsumption = (double)gasConsumption - (double)((double)digit * 100000.0);
	  calcValue = ((double)gasConsumption / 10000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 10.000 digit of gas consumption */
	  gasConsumption = (double)gasConsumption - (double)((double)digit * 10000.0);
	  calcValue = ((double)gasConsumption / 1000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 1.000 digit of gas consumption */
	  gasConsumption = (double)gasConsumption - (double)((double)digit * 1000.0);
	  calcValue = ((double)gasConsumption / 100.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 100 digit of gas consumption */
	  sendTxd0('.');
	  gasConsumption = (double)gasConsumption - (double)((double)digit * 100.0);
	  calcValue = ((double)gasConsumption / 10.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 10 digit of gas consumption */
	  gasConsumption = (double)gasConsumption - (double)((double)digit * 10.0);
	  calcValue = ((double)gasConsumption / 1.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 1 digit of gas consumption */
   } //if (ini_rcvd_yet)
   else
   {
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('.');
	  sendTxd0('-');
	  sendTxd0('-');
   } //else of if (ini_rcvd_yet)

   sendTxd0('C');
   sendTxd0('B');
   sendTxd0('M');

   if (ini_rcvd_yet)
   { 
  	  calcValue = ((double)waterConsumption / 100000000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 100.000.000 digit of water consumption */
	  waterConsumption = (double)waterConsumption - (double)((double)digit * 100000000.0);
	  calcValue = ((double)waterConsumption / 10000000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 10.000.000 digit of water consumption */
	  waterConsumption = (double)waterConsumption - (double)((double)digit * 10000000.0);
	  calcValue = ((double)waterConsumption / 1000000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 1.000.000 digit of water consumption */
	  waterConsumption = (double)waterConsumption - (double)((double)digit * 1000000.0);
	  calcValue = ((double)waterConsumption / 100000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 100.000 digit of water consumption */
	  waterConsumption = (double)waterConsumption - (double)((double)digit * 100000.0);
	  calcValue = ((double)waterConsumption / 10000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 10.000 digit of water consumption */
	  sendTxd0('.');
	  waterConsumption = (double)waterConsumption - (double)((double)digit * 10000.0);
	  calcValue = ((double)waterConsumption / 1000.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 1.000 digit of water consumption */
	  waterConsumption = (double)waterConsumption - (double)((double)digit * 1000.0);
	  calcValue = ((double)waterConsumption / 100.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 100 digit of water consumption */
	  waterConsumption = (double)waterConsumption - (double)((double)digit * 100.0);
	  calcValue = ((double)waterConsumption / 10.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 10 digit of water consumption */
	  waterConsumption = (double)waterConsumption - (double)((double)digit * 10.0);
	  calcValue = ((double)waterConsumption / 1.0);
	  digit     = (unsigned char)calcValue;
	  sendTxd0(digit+0x30); /* send 1 digit of water consumption */
   } //if (ini_rcvd_yet)
   else
   {
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('.');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
	  sendTxd0('-');
   } //else of if (ini_rcvd_yet)

   sendTxd0('C');
   sendTxd0('B');
   sendTxd0('M');
} //else of if (b_VERSIONsendREQ)
sendTxd0('e'); /* send end character 'e' */
}

#pragma interrupt UART0_Receive_int
void UART0_Receive_int(void)
{
unsigned char rcvd_byte;
	ir_s0ric = 0; /* clear reception flag */
	rs232_rcv.word = u0rb; /* get UART-receive-buffer and flags */
	rcvd_byte = rs232_rcv.byte.rs232_rcv_data;
	if (rcvd_byte == 'v')
	{
	   b_VERSIONsendREQ = 1;
	}
	else if (rcvd_byte == 's')
	{
		startIndicator = 1;
		positionPointer = 0;
	}
	else if (rcvd_byte == 'e')
	{
		startIndicator = 0;
		positionPointer = 0;	
		ini_rcvd_yet = 1;
	}
	else if (startIndicator == 1)
	{
		rs232_rcv_array[positionPointer] = rcvd_byte;		
		positionPointer++;
	}
	re_u0c1 = 1; /* enable reception again ? */
}
/***********************************************************************************/
